@@ -76,9 +76,11 @@ if(!is.null(spobj)) {
      }
    if(!is.null(a$Y)&&!is.null(a$X)) {data=a$Y ; X=a$X }
 }
+
 ###### setting nugget if missing
 if(!bivariate)
-   if(!sum(names(unlist(append(start,fixed)))=="nugget")) fixed$nugget=0
+   if(!sum(substr(names(unlist(append(start,fixed))),1,6)=="nugget")) fixed$nugget=0
+
 ###############################################################
 ###############################################################  
 if(!bivariate){
@@ -86,7 +88,7 @@ if(model %in% c("Weibull","Poisson","Binomial","Gamma","LogLogistic",
         "BinomialNeg","Bernoulli","Geometric","Gaussian_misp_Poisson","Binary_misp_BinomialNeg",
         'PoissonZIP','Gaussian_misp_PoissonZIP','BinomialNegZINB',
         'PoissonZIP1','Gaussian_misp_PoissonZIP1','BinomialNegZINB1',
-        "PoissonGamma","PoissonGammaZIP",
+        "PoissonGamma","PoissonGammaZIP","PoissonGammaZIP1",
         'Beta2','Kumaraswamy2','Beta','Kumaraswamy')) {
 if(!is.null(start$sill)) stop("sill parameter must not be considered for this model\n")    
 if(is.null(fixed$sill)) fixed$sill=1
